@@ -40,7 +40,8 @@ def run_bench(name, use_gevent=False):
 
 def bench_wrapper(use_gevent=False):
     print '=' * 80
-    print 'Running benchmark with Logbook %s (gevent enabled=%s)' % (version, use_gevent)
+    print 'Running benchmark with Logbook %s (gevent enabled=%s)' % \
+        (version, use_gevent)
     print '-' * 80
     os.chdir(bench_directory)
     for bench in list_benchmarks():
