@@ -11,6 +11,14 @@ def test_exc_info_when_no_exceptions_exist(logger):
             logger.debug('message', exc_info=True)
     assert 'Traceback' not in captured.getvalue()
 
+
+def test_exc_info_false():
+    with logbook.handlers.TestHandler() as handler:
+        logbook.debug('message here', exc_info=False)
+    [record] = handler.records
+    assert not record.formatted_exception
+
+
 def test_extradict(active_handler, logger):
     logger.warn('Test warning')
     record = active_handler.records[0]
@@ -24,6 +32,7 @@ def test_calling_frame(active_handler, logger):
     logger.warn('test')
     assert active_handler.records[0].calling_frame == sys._getframe()
 
+
 def test_frame_correction(active_handler, logger):
     def inner():
         logger.warn('test', frame_correction=+1)
@@ -31,6 +40,7 @@ def test_frame_correction(active_handler, logger):
     inner()
     assert active_handler.records[0].calling_frame == sys._getframe()
 
+
 def test_dispatcher(active_handler, logger):
     logger.warn('Logbook is too awesome for stdlib')
     assert active_handler.records[0].dispatcher == logger
