@@ -11,7 +11,8 @@ def test_regex_matching(active_handler, logger):
 def test_test_handler_cache(active_handler, logger):
     logger.warn('First line')
     assert len(active_handler.formatted_records) == 1
-    cache = active_handler.formatted_records  # store cache, to make sure it is identifiable
+    # store cache, to make sure it is identifiable
+    cache = active_handler.formatted_records
     assert len(active_handler.formatted_records) == 1
     assert cache is active_handler.formatted_records
     logger.warn('Second line invalidates cache')
